/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
*  This is source code for the PSoC 4 BLE Find Me Profile Target example project.
*
* Note:
*
* Owners:
*  OLEG@CYPRESS.COM, KRIS@CYPRESS.COM
*
* Related Document:
*
* Hardware Dependency:
*  1. PSoC 4 BLE device
*  2. CY8CKIT-042 BLE Pioneer Kit
*
* Code Tested With:
*  1. PSoC Creator 3.1
*  2. ARM CM3-RVDS
*  3. ARM CM3-GCC
*  4. ARM CM3-MDK
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include <project.h>
#include <stdbool.h>
#include "common.h"


void SetupLocalName()
{
    
    char devName[] = "SimpleBLE-XXXXXX";
    uint32 addr = (CYBLE_SFLASH_DIE_X_REG & CYBLE_SFLASH_DIE_X_MASK) |
                    ((uint32)(((uint32)CYBLE_SFLASH_DIE_Y_REG) & ((uint32)CYBLE_SFLASH_DIE_Y_MASK)) <<
                        CYBLE_SFLASH_DIE_X_BITS) |
                    ((uint32)(((uint32)CYBLE_SFLASH_DIE_WAFER_REG) & ((uint32)CYBLE_SFLASH_DIE_WAFER_MASK)) <<
                        CYBLE_SFLASH_DIE_XY_BITS) |
                    ((uint32)(((uint32)CYBLE_SFLASH_DIE_LOT_REG) & ((uint32)CYBLE_SFLASH_DIE_LOT_MASK)) <<
                        CYBLE_SFLASH_DIE_XYWAFER_BITS);
    
    devName[10] = ((addr >> 20) & 0x0F) + 'A';
    devName[11] = ((addr >> 16) & 0x0F) + 'A';
    devName[12] = ((addr >> 12) & 0x0F) + 'A';
    devName[13] = ((addr >> 8) & 0x0F) + 'A';
    devName[14] = ((addr >> 4) & 0x0F) + 'A';
    devName[15] = ((addr >> 0) & 0x0F) + 'A';       
    
    CyBle_GapSetLocalName(devName);
}

uint8 I2CReadRegister(uint8 slaveAddr, uint8 regAddr)
{
    uint8 ret = 0xFF;
    
    I2C_I2CMasterClearStatus();  
    I2C_I2CMasterSendStart(slaveAddr, I2C_I2C_WRITE_XFER_MODE);
    I2C_I2CMasterWriteByte(regAddr);
    I2C_I2CMasterSendRestart(slaveAddr, I2C_I2C_READ_XFER_MODE);
    ret = I2C_I2CMasterReadByte(I2C_I2C_NAK_DATA);
    I2C_I2CMasterSendStop();
    
    return ret;
}

void I2CWriteRegister(uint8 slaveAddr, uint8 regAddr, uint8 writeData)
{
    uint8 buf[2] = {regAddr, writeData};
    
    I2C_I2CMasterClearStatus();
    I2C_I2CMasterWriteBuf(slaveAddr, buf, 2, I2C_I2C_MODE_COMPLETE_XFER);
    
    while (! (I2C_I2CMasterStatus() & I2C_I2C_MSTAT_WR_CMPLT));
}

#define STTS751_I2C_SLAVE_ADDR (0x72 >> 1)
#define STTS751_TEMP_HI_BYTE    0x00
#define STTS751_TEMP_LOW_BYTE   0x02
#define STTS751_CONFIGRATION    0x03
#define STTS751_MASK_BIT       (1 << 7)
#define STTS7551_STOP_BIT      (1 << 6)

uint16 ReadTemp()
{
    uint8 hi = I2CReadRegister(STTS751_I2C_SLAVE_ADDR, STTS751_TEMP_HI_BYTE);
    uint8 low = I2CReadRegister(STTS751_I2C_SLAVE_ADDR, STTS751_TEMP_LOW_BYTE);
    return ((uint16)hi << 8) | low;
}

void UpdateGattDB()
{
    CYBLE_GATT_HANDLE_VALUE_PAIR_T handleVal;
    uint16 val = ReadTemp();
    handleVal.attrHandle = CYBLE_THERMALMONITOR_THERMALSENSOR_CHAR_HANDLE;
    handleVal.value.val = (uint8*)&val;
    handleVal.value.len = 2;
    CyBle_GattsWriteAttributeValue(&handleVal, 0, &cyBle_connHandle, CYBLE_GATT_DB_LOCALLY_INITIATED);   
}


void StartThermSensor()
{
    I2CWriteRegister(STTS751_I2C_SLAVE_ADDR, STTS751_CONFIGRATION, STTS751_MASK_BIT);
}

void StopThermSensor()
{
    I2CWriteRegister(STTS751_I2C_SLAVE_ADDR, STTS751_CONFIGRATION, (STTS751_MASK_BIT | STTS7551_STOP_BIT));
}
 
void StackEventHandler(uint32 event, void *eventParam)
{
    switch(event)
    {
        case CYBLE_EVT_STACK_ON:
            SetupLocalName();
            StopThermSensor();
            CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);            
            break;
        case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:
            StopThermSensor();
            CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
            break;
        case CYBLE_EVT_GAP_DEVICE_CONNECTED:
            {
                static CYBLE_GAP_CONN_UPDATE_PARAM_T Param;
                Param.connIntvMax = 1000;
                Param.connIntvMin = 800;
                Param.supervisionTO = 500;
                CyBle_L2capLeConnectionParamUpdateRequest(cyBle_connHandle.bdHandle, &Param);
            }
            StartThermSensor();
            break;
        case CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
            if(CyBle_GetState() == CYBLE_STATE_DISCONNECTED)
            {
                CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
            }
            break;
    }
}



int main()
{

    CyGlobalIntEnable;
    
    I2C_scl_SetDriveMode(I2C_scl_DM_RES_UP);
    I2C_sda_SetDriveMode(I2C_sda_DM_RES_UP);
    I2C_Start();
    StopThermSensor();
    
    CyBle_Start(StackEventHandler);
 
    for(;;)
    {

        CYBLE_BLESS_STATE_T blessState;
        uint8 intrStatus;
        
        if( CyBle_GetState() == CYBLE_STATE_CONNECTED )
        {
            UpdateGattDB();
        }

        /* Single API call to service all the BLE stack events. Must be
         * called at least once in a BLE connection interval */
        CyBle_ProcessEvents();
        
        /* Configure BLESS in Deep-Sleep mode */
        CyBle_EnterLPM(CYBLE_BLESS_DEEPSLEEP);
        
        /* Prevent interrupts while entering system low power modes */
        intrStatus = CyEnterCriticalSection();
        
        /* Get the current state of BLESS block */
        blessState = CyBle_GetBleSsState();
        
        /* If BLESS is in Deep-Sleep mode or the XTAL oscillator is turning on, 
         * then PSoC 4 BLE can enter Deep-Sleep mode (1.3uA current consumption) */
        if(blessState == CYBLE_BLESS_STATE_ECO_ON || 
            blessState == CYBLE_BLESS_STATE_DEEPSLEEP)
        {
            CySysPmDeepSleep();
        }
        else if(blessState != CYBLE_BLESS_STATE_EVENT_CLOSE)
        {
            /* If BLESS is active, then configure PSoC 4 BLE system in 
             * Sleep mode (~1.6mA current consumption) */
            CySysPmSleep();
         }
        else
        {
            /* Keep trying to enter either Sleep or Deep-Sleep mode */    
        }
        CyExitCriticalSection(intrStatus);
        
        /* BLE link layer timing interrupt will wake up the system from Sleep 
         * and Deep-Sleep modes */
    }
}

